/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_MC9S12G128.c for MC9S12G128 CPU and 
          CodeWarrior compiler
          
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency     */
  #define OS_FSYS 50000000uL        /* Assume 50MHz                 */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER (OS_FSYS/2) /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

#define OS_UART_USED 0 // Not supported with this evalboard

/****** End of configurable options *********************************/

#define OS_TIMER_RELOAD       (OS_PCLK_TIMER / OS_TICK_FREQ)

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/

#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define CPMUPROT          (*(volatile OS_U8*)(0x02FB))
#define CPMUCLKS          (*(volatile OS_U8*)(0x0039))
#define CPMUSYNR          (*(volatile OS_U8*)(0x0034))
#define CPMUREFDIV        (*(volatile OS_U8*)(0x0035))
#define CPMUPOSTDIV       (*(volatile OS_U8*)(0x0036))
#define CPMUPLL           (*(volatile OS_U8*)(0x003A))
#define CPMUFLG           (*(volatile OS_U8*)(0x0037))
#define CPMUOSC           (*(volatile OS_U8*)(0x02FA))

#define TIM_BASE_ADDR     0x40
#define TIOS              (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x00))
#define OC7M              (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x02))
#define TCNT              (*(volatile OS_U16*)(TIM_BASE_ADDR + 0x04))
#define TSCR1             (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x06))
#define TTOV              (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x07))
#define TCTL1             (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x08))
#define TCTL2             (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x09))
#define TIE               (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x0C))
#define TSCR2             (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x0D))
#define TFLG1             (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x0E))
#define TC0               (*(volatile OS_U16*)(TIM_BASE_ADDR + 0x10))
#define TC7               (*(volatile OS_U16*)(TIM_BASE_ADDR + 0x1E))
#define PACTL             (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x20))
#define OCPD              (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x2C))
#define PTPSR             (*(volatile OS_U8 *)(TIM_BASE_ADDR + 0x2E))

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#pragma CODE_SEG __NEAR_SEG NON_BANKED
/*********************************************************************
*
*       OS_ISR_Tick()
*       The OS system timer interrupt
*/
__interrupt void __near OS_ISR_Tick (void) {
  TFLG1 = 0x80;   // Reset interrupt request flag 
  OS_EnterNestableInterrupt();
  OS_TICK_Handle();
  OS_LeaveNestableInterrupt();
}
#pragma CODE_SEG DEFAULT

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_IncDI();
  //
  // Initialize PLL
  //
  CPMUPROT    = 0x26;           // Disable protection of clock register
  CPMUCLKS    = 0x40;           // Reset
  CPMUCLKS    = 0x80;           // Enable the PLL to allow write to divider registers
  CPMUREFDIV  = 0x07;           // Set devider
  CPMUSYNR    = 0x58;           // Set the multiplier register 
  CPMUPOSTDIV = 0x00;           // Set the post divider register
  CPMUOSC     = 0x80;           // Enable the oscillator
  CPMUPLL     = 0x00;           // Set the PLL frequency modulation
  while((CPMUFLG & 0x01) == 0); // Wait until the oscillator is qualified by the PLL
  CPMUPROT    = 0x00;           // Enable protection of clock register
  //
  // Initialize timer for embOS
  //
  TSCR1 = 0x08;                 // Enable precision timer
  OCPD  = 0xFF;                 // Disables the timer channel port 
  PACTL = 0x0C;                 // Use PACLK/65536 as timer counter clock frequency
  OC7M  = 0x80;                 // Output Compare 7 Mask
  TIOS  = 0x80;                 // Timer Input Capture/Output Compare Select 
  TCTL1 = 0xC0;                 // Timer Control Register 1
  TTOV  = 0x80;                 // Timer Toggle On Overlow Register 1
  TSCR2 = 0x08;                 // Timer System Control Register 2
  TFLG1 = 0xFF;                 // Clear flags
  PTPSR = 0x00;                 // Prescale Factor = 1
  TC7   = OS_TIMER_RELOAD;      // Set Reload value
  TIE   = 0x80;                 // Enable timer interrupt
  TSCR1 = 0x80;                 // Enable timer
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_GetTime32();
  Cnt  = TCNT;
  //
  // Check if timer interrupt is pending
  //
  if (TFLG1 & 0x80) {  
    Cnt = TCNT;
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if OS_UART_USED
#else  /* UART for communication not used, define dummy functions */

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}
#endif

/****** End Of File *************************************************/
