/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Freescale MC9S12XEG128 (custom hardware)
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define LED0_BIT        3  // (Pin 0)
#define LED1_BIT        2  // (Pin 1)
#define LED_MASK_ALL    ((1u << LED0_BIT) | (1u << LED1_BIT))

#define PORTP_BASE_ADDR 0x250
#define PTP             (*(volatile unsigned char*)(PORTP_BASE_ADDR + 0x08))
#define PTIP            (*(volatile unsigned char*)(PORTP_BASE_ADDR + 0x09))
#define DDRP            (*(volatile unsigned char*)(PORTP_BASE_ADDR + 0x0A))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  DDRP |= LED_MASK_ALL;  // Set port pins to output
  PTP  &= ~LED_MASK_ALL;  // Turn off Leds
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PTP |= (1u << LED0_BIT);
  } else if (Index == 1) {
    PTP |= (1u << LED1_BIT);
  } 
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PTP &= ~(1u << LED0_BIT);
  } else if (Index == 1) {
    PTP &= ~(1u << LED1_BIT);
  } 
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if ((PTIP & (1u << LED0_BIT)) == 0) {
      PTP |= (1u << LED0_BIT);
    } else {
      PTP &= ~(1u << LED0_BIT);
    }
  } else if (Index == 1) {
    if ((PTIP & (1u << LED1_BIT)) == 0) {
      PTP |= (1u << LED1_BIT);
    } else {
      PTP &= ~(1u << LED1_BIT);
    }
  }
}

/****** End Of File *************************************************/
