/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_MC9S12XEG128.c for MC9S12XEG128 CPU and 
          CodeWarrior compiler
          
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency     */
  #define OS_FSYS 100000000uL       /* Assume 100MHz                */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER (OS_FSYS/2) /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

#define OS_UART_USED 0 // Not supported with this evalboard

/****** End of configurable options *********************************/

#define OS_MICRO_TIMER0_VALUE 8u
#define OS_TIMER_RELOAD       (OS_PCLK_TIMER / OS_TICK_FREQ / OS_MICRO_TIMER0_VALUE)

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/

#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define ECRG_BASE_ADDR    0x34
#define SYNR              (*(volatile OS_U8*)(ECRG_BASE_ADDR + 0x00))
#define REFDV             (*(volatile OS_U8*)(ECRG_BASE_ADDR + 0x01))
#define POSTDIV           (*(volatile OS_U8*)(ECRG_BASE_ADDR + 0x02))
#define CRGFLG            (*(volatile OS_U8*)(ECRG_BASE_ADDR + 0x03))
#define CLKSEL            (*(volatile OS_U8*)(ECRG_BASE_ADDR + 0x05))
#define PLLCTL            (*(volatile OS_U8*)(ECRG_BASE_ADDR + 0x06))

#define PIT_BASE_ADDR     0x340
#define PITCFLMT          (*(volatile OS_U8*) (PIT_BASE_ADDR + 0x00))
#define PITCE             (*(volatile OS_U8*) (PIT_BASE_ADDR + 0x02))
#define PITMUX            (*(volatile OS_U8*) (PIT_BASE_ADDR + 0x03))
#define PITINTE           (*(volatile OS_U8*) (PIT_BASE_ADDR + 0x04))
#define PITTF             (*(volatile OS_U8*) (PIT_BASE_ADDR + 0x05))
#define PITMTLD0          (*(volatile OS_U8*) (PIT_BASE_ADDR + 0x06))
#define PITLD0            (*(volatile OS_U16*)(PIT_BASE_ADDR + 0x08))
#define PITCNT0           (*(volatile OS_U16*)(PIT_BASE_ADDR + 0x0A))
                                                                     
/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#pragma CODE_SEG __NEAR_SEG NON_BANKED
/*********************************************************************
*
*       OS_ISR_Tick()
*       The OS system timer interrupt
*/
__interrupt void __near OS_ISR_Tick (void) {
  PITTF = 0x01u; // Reset interrupt request flag
  OS_EnterNestableInterrupt();
  OS_TICK_Handle();
  OS_LeaveNestableInterrupt();
}
#pragma CODE_SEG DEFAULT

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_IncDI();
  //
  // Initialize PLL, F(PLL) = (2 x F(OSC) * ((SYNDIV + 1) / (REFDIV + 1))) / (2 x POSTDIV)
  //
  SYNR    = (3u << 6) | 24;     
  REFDV   = (1u << 6) | 3;
  POSTDIV = 0;
  PLLCTL  = 0xC1;
  while ((CRGFLG & 0x08) == 0); // Wait until PLL is stable
  CLKSEL = 0x80;                // Select PLL as clock source
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  PITCFLMT &= ~0x80; // Disable PIT
  PITCE    |= 0x01;  // Enable channel 0
  PITMUX   &= ~0x01; // Channel 0 is connected to micro timer 0
  PITMTLD0  = OS_MICRO_TIMER0_VALUE - 1; 
  PITLD0    = OS_TIMER_RELOAD - 1;       
  PITINTE  |= 0x01;  // Enable interrupt
  PITCFLMT |= 0x80;  // Enable PIT
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_GetTime32();
  Cnt  = OS_TIMER_RELOAD - PITCNT0;
  //
  // Check if timer interrupt is pending
  //
  if (PITTF & 0x01) {  
    Cnt = OS_TIMER_RELOAD - PITCNT0;
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= OS_MICRO_TIMER0_VALUE;     // Eliminate effect of prescaler 
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if OS_UART_USED
#else  /* UART for communication not used, define dummy functions */

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}
#endif

/****** End Of File *************************************************/
